/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: 8100_CMPSTPO.sql

Descrio:
Calcula o limite de seguranca do produto considerando ou no os dias no teis.
Tambm retorna o nmero de dias de limite de segurana cadastrado para o produto.

Criao:
08/07/2000 Maria de Ftima e Ederson Melquades da Cunha
*/

IF EXISTS (SELECT
              *
           FROM
              SYSOBJECTS O,
              SYSUSERS U
           WHERE
              O.NAME = 'LIMISEGU' AND
              TYPE   = 'P'        AND
              U.UID  = O.UID      AND
              U.NAME = USER)
   DROP PROCEDURE LIMISEGU;
   GO

CREATE PROCEDURE LIMISEGU @P_CDFILIAL  VARCHAR(4),
                          @P_CDARVPROD VARCHAR(20),
                          @P_DTENTREGA DATETIME,
                          @P_CDPERIENTR VARCHAR(2),
                          @P_NRLIMISEGU NUMERIC OUTPUT,
                          @P_NRLIMISEGCAD VARCHAR(3) OUTPUT
AS

   DECLARE @V_IDDNULIMSEG  VARCHAR(1);
   DECLARE @V_NRLIMISEGU   NUMERIC(5);
   DECLARE @V_NRCONTA      NUMERIC(5);
   DECLARE @V_DTENTREGA    DATETIME;

BEGIN
   SET @V_NRLIMISEGU = 0;
   SET @V_DTENTREGA  = @P_DTENTREGA;

   SELECT
      @V_NRLIMISEGU   = MAX(CONVERT(NUMERIC, NRLIMISEGU)),
      @P_NRLIMISEGCAD = MAX(NRLIMISEGU)
   FROM
      LIMISEGPROD,
      PRODUTO
   WHERE
      (LIMISEGPROD.CDPRODUTO = PRODUTO.CDPRODUTO) AND
      (LIMISEGPROD.CDFILIAL  = @P_CDFILIAL)       AND
      (PRODUTO.CDARVPROD     = (SELECT
                                   MAX(CDARVPROD)
                                FROM
                                   LIMISEGPROD A,
                                   PRODUTO B
                                WHERE
                                   (A.CDPRODUTO = B.CDPRODUTO) AND
                                   (A.CDFILIAL  = @P_CDFILIAL) AND
                                   (CHARINDEX(B.CDARVPROD, @P_CDARVPROD) = 1)));

   IF @V_NRLIMISEGU > 0 BEGIN
      /* Busca o parmetro que indica se considera dias no teis */
      SET @V_IDDNULIMSEG = NULL;
      SELECT
         @V_IDDNULIMSEG = MAX(PERIENTR.IDDNULIMSEGP)
      FROM
         PERIENTR
      WHERE
         PERIENTR.CDPERIENTR = @P_CDPERIENTR;
      IF @V_IDDNULIMSEG IS NULL BEGIN
         SELECT
            @V_IDDNULIMSEG = ISNULL(MAX(PARASUPRI.IDDNULIMSEG), 'N')
         FROM
            PARASUPRI,
            FILIAL
         WHERE
            (FILIAL.CDFILIAL    = @P_CDFILIAL) AND
            (PARASUPRI.CDFILIAL = ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL));
      END;

      /* Cursor com dias teis no intervalo de um ano a partir da data de entrega */
      DECLARE C_MAIORDATA CURSOR FOR
         SELECT
            DTDATA
         FROM
            DATA
         WHERE
            (DTDATA BETWEEN @P_DTENTREGA AND DATEADD(DD, 364, @P_DTENTREGA)) AND
            ((@V_IDDNULIMSEG = 'N') OR ((@V_IDDNULIMSEG = 'S') AND
                                        (DTDATA NOT IN (SELECT
                                                           DTFERICOMP
                                                        FROM
                                                           FERICOMP
                                                        WHERE
                                                           (CDFILIAL   = @P_CDFILIAL)  AND
                                                           (DTFERICOMP > @P_DTENTREGA) AND
                                                            NOT EXISTS (SELECT 1
                                                                        FROM CALECOMP
                                                                        WHERE CALECOMP.CDFILIAL   = @P_CDFILIAL
                                                                          AND CALECOMP.DTENTREGA  = DTFERICOMP
                                                                          AND CALECOMP.CDPERIENTR = @P_CDPERIENTR) ))));
      OPEN C_MAIORDATA;
      SET @V_NRCONTA = 0;
      WHILE @V_NRCONTA <= @V_NRLIMISEGU BEGIN
         /* Busca a ltima data ate o limite de segurana */
         FETCH NEXT FROM C_MAIORDATA INTO @V_DTENTREGA;
         IF @@FETCH_STATUS <> 0 BREAK;
         SET @V_NRCONTA = @V_NRCONTA + 1;
      END;
      CLOSE      C_MAIORDATA;
      DEALLOCATE C_MAIORDATA;
      SET @V_NRLIMISEGU = DATEDIFF(DD, @P_DTENTREGA, @V_DTENTREGA);
   END
   ELSE BEGIN
      SET @V_NRLIMISEGU = 0;
   END;
   SET @P_NRLIMISEGU = @V_NRLIMISEGU;
END;
GO